@@ -532,6 +532,7 @@ PHP_FUNCTION(stomp_connect)
 	if (stomp->status) {
 		stomp_frame_t *res;
 		stomp_frame_t frame = {0};
+		int send_status;
  
 		INIT_FRAME(frame, "CONNECT");
 		if (!username) {
@@ -542,18 +543,34 @@ PHP_FUNCTION(stomp_connect)
 			password = STOMP_G(default_password);
 			password_len = strlen(password);
 		}
-		zend_hash_add(frame.headers, "login", sizeof("login"), username, username_len + 1, NULL);
-		zend_hash_add(frame.headers, "passcode", sizeof("passcode"), password, password_len + 1, NULL);
+
+		/*
+		 * Per Stomp 1.1 "login" and "passcode" are optional. (Also this fix makes test pass against RabbitMQ)
+		 */
+        if (username_len > 0) {
+			zend_hash_add(frame.headers, "login", sizeof("login"), username, username_len + 1, NULL);
+        }
+
+		if (password_len > 0) {
+			zend_hash_add(frame.headers, "passcode", sizeof("passcode"), password, password_len + 1, NULL);
+		}
  
 		if (NULL != headers) {
 			FRAME_HEADER_FROM_HASHTABLE(frame.headers, Z_ARRVAL_P(headers));
 		}
 
-		stomp_send(stomp, &frame TSRMLS_CC);
+		send_status = stomp_send(stomp, &frame TSRMLS_CC);
 		CLEAR_FRAME(frame);
+		if (0 == send_status) {
+			zval *excobj = zend_throw_exception_ex(stomp_ce_exception, stomp->errnum TSRMLS_CC, stomp->error);
+			if (stomp->error_details) {
+				zend_update_property_string(stomp_ce_exception, excobj, "details", sizeof("details")-1, stomp->error_details TSRMLS_CC);
+			}
+			return;
+		}
 
 		/* Retreive Response */
-		res = stomp_read_frame(stomp);
+		res = stomp_read_frame_ex(stomp, 0);
 		if (NULL == res) {
 			STOMP_ERROR(0, PHP_STOMP_ERR_SERVER_NOT_RESPONDING);
 		} else if (0 == strncmp("ERROR", res->command, sizeof("ERROR") - 1)) {
@@ -788,10 +805,13 @@ PHP_FUNCTION(stomp_subscribe)
 		FRAME_HEADER_FROM_HASHTABLE(frame.headers, Z_ARRVAL_P(headers));
 	}
 
+	/* Add the ack:client (if not overwritten through headers)*/
+	if (!zend_hash_exists(frame.headers, "ack", sizeof("ack"))) {
+		zend_hash_add(frame.headers, "ack", sizeof("ack"), "client", sizeof("client"), NULL);
+	}
 	/* Add the destination */
-	zend_hash_add(frame.headers, "ack", sizeof("ack"), "client", sizeof("client"), NULL);
 	zend_hash_add(frame.headers, "destination", sizeof("destination"), destination, destination_length + 1, NULL);
-	zend_hash_add(frame.headers, "activemq.prefetchSize", sizeof("activemq.prefetchSize"), "1", sizeof("1"), NULL); 
+	//zend_hash_add(frame.headers, "activemq.prefetchSize", sizeof("activemq.prefetchSize"), "1", sizeof("1"), NULL); 
 
 	if (stomp_send(stomp, &frame TSRMLS_CC) > 0) {
 		success = stomp_valid_receipt(stomp, &frame);
@@ -910,7 +930,7 @@ PHP_FUNCTION(stomp_read_frame)
 	}
 
 
-	if ((res = stomp_read_frame(stomp))) {
+	if ((res = stomp_read_frame_ex(stomp, 1))) {
 		zval *headers = NULL;
 
 		if (0 == strncmp("ERROR", res->command, sizeof("ERROR") - 1)) {
